@@ -65,8 +65,7 @@ class TerminalWindow: NSWindow {
         // fallback to original centering behavior
         setInitialWindowPosition(
             x: config.windowPositionX,
-            y: config.windowPositionY,
-            windowDecorations: config.windowDecorations)
+            y: config.windowPositionY)
 
         // If our traffic buttons should be hidden, then hide them
         if config.macosWindowButtons == .hidden {
@@ -124,6 +123,12 @@ class TerminalWindow: NSWindow {
         } else {
             tabBarDidDisappear()
         }
+        viewModel.isMainWindow = true
+    }
+
+    override func resignMain() {
+        super.resignMain()
+        viewModel.isMainWindow = false
     }
 
     override func mergeAllWindows(_ sender: Any?) {
@@ -164,9 +169,16 @@ class TerminalWindow: NSWindow {
 
     /// Returns true if there is a tab bar visible on this window.
     var hasTabBar: Bool {
+        // TODO: use titlebarView to find it instead
         contentView?.firstViewFromRoot(withClassName: "NSTabBar") != nil
     }
 
+    var hasMoreThanOneTabs: Bool {
+        /// accessing ``tabGroup?.windows`` here
+        /// will cause other edge cases, be careful
+        (tabbedWindows?.count ?? 0) > 1
+    }
+
     func isTabBar(_ childViewController: NSTitlebarAccessoryViewController) -> Bool {
         if childViewController.identifier == nil {
             // The good case
@@ -260,7 +272,7 @@ class TerminalWindow: NSWindow {
         button.isBordered = false
         button.allowsExpansionToolTips = true
         button.toolTip = "Reset Zoom"
-        button.contentTintColor = .controlAccentColor
+        button.contentTintColor = isMainWindow ? .controlAccentColor : .secondaryLabelColor
         button.state = .on
         button.image = NSImage(named:"ResetZoom")
         button.frame = NSRect(x: 0, y: 0, width: 20, height: 20)
@@ -277,6 +289,12 @@ class TerminalWindow: NSWindow {
             // Whenever we change the window title we must also update our
             // tab title if we're using custom fonts.
             tab.attributedTitle = attributedTitle
+            /// We also needs to update this here, just in case
+            /// the value is not what we want
+            ///
+            /// Check ``titlebarFont`` down below
+            /// to see why we need to check `hasMoreThanOneTabs` here
+            titlebarTextField?.usesSingleLineMode = !hasMoreThanOneTabs
         }
     }
 
@@ -286,6 +304,12 @@ class TerminalWindow: NSWindow {
             let font = titlebarFont ?? NSFont.titleBarFont(ofSize: NSFont.systemFontSize)
 
             titlebarTextField?.font = font
+            /// We check `hasMoreThanOneTabs` here because the system
+            /// may copy this setting to the tab’s text field at some point(e.g. entering/exiting fullscreen),
+            /// which can cause the title to be vertically misaligned (shifted downward).
+            ///
+            /// This behaviour is the opposite of what happens in the title bar’s text field, which is quite odd...
+            titlebarTextField?.usesSingleLineMode = !hasMoreThanOneTabs
             tab.attributedTitle = attributedTitle
         }
     }
@@ -400,7 +424,7 @@ class TerminalWindow: NSWindow {
         return derivedConfig.backgroundColor.withAlphaComponent(alpha)
     }
 
-    private func setInitialWindowPosition(x: Int16?, y: Int16?, windowDecorations: Bool) {
+    private func setInitialWindowPosition(x: Int16?, y: Int16?) {
         // If we don't have an X/Y then we try to use the previously saved window pos.
         guard let x, let y else {
             if (!LastWindowPosition.shared.restore(self)) {
@@ -416,19 +440,14 @@ class TerminalWindow: NSWindow {
             return
         }
 
-        // Convert top-left coordinates to bottom-left origin using our utility extension
-        let origin = screen.origin(
-            fromTopLeftOffsetX: CGFloat(x),
-            offsetY: CGFloat(y),
-            windowSize: frame.size)
+        // We have an X/Y, use our controller function to set it up.
+        guard let terminalController else {
+            center()
+            return
+        }
         
-        // Clamp the origin to ensure the window stays fully visible on screen
-        var safeOrigin = origin
-        let vf = screen.visibleFrame
-        safeOrigin.x = min(max(safeOrigin.x, vf.minX), vf.maxX - frame.width)
-        safeOrigin.y = min(max(safeOrigin.y, vf.minY), vf.maxY - frame.height)
-        
-        setFrameOrigin(safeOrigin)
+        let frame = terminalController.adjustForWindowPosition(frame: frame, on: screen)
+        setFrameOrigin(frame.origin)
     }
 
     private func hideWindowButtons() {
@@ -467,6 +486,7 @@ extension TerminalWindow {
     class ViewModel: ObservableObject {
         @Published var isSurfaceZoomed: Bool = false
         @Published var hasToolbar: Bool = false
+        @Published var isMainWindow: Bool = true
     }
 
     struct ResetZoomAccessoryView: View {
@@ -488,7 +508,7 @@ extension TerminalWindow {
                 VStack {
                     Button(action: action) {
                         Image("ResetZoom")
-                            .foregroundColor(.accentColor)
+                            .foregroundColor(viewModel.isMainWindow ? .accentColor : .secondary)
                     }
                     .buttonStyle(.plain)
                     .help("Reset Split Zoom")
