@@ -13,4 +13,4 @@ docker run \
   --entrypoint "xvfb-run" \
   $IMAGE \
   --server-args="-screen 0, 1600x900x24" \
-  /entrypoint.sh $@
+  /entrypoint.sh "$@"
